@@ -2,21 +2,24 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 0, 8)
+__version__ = (0, 0, 9)
 __all__ = [
     "create_cookie", "create_morsel", "to_cookie", "to_morsel", 
-    "cookie_to_morsel", "morsel_to_cookie", "cookies_str_to_dict", 
-    "cookies_dict_to_str", "extract_cookies", "update_cookies", 
+    "cookie_to_morsel", "morsel_to_cookie", "cookies_to_dict", 
+    "cookies_to_str", "extract_cookies", "update_cookies", 
     "iter_resp_cookies", 
 ]
 
 from calendar import timegm
-from collections.abc import Buffer, Iterable, Iterator, Mapping, Sequence
+from collections.abc import (
+    Buffer, Callable, Container, Iterable, Iterator, Mapping, Sequence, 
+)
 from copy import copy
 from datetime import datetime
 from functools import partial
 from http.cookiejar import Cookie, CookieJar
 from http.cookies import Morsel, SimpleCookie
+from posixpath import commonpath
 from re import compile as re_compile
 from time import gmtime, strftime, strptime, time
 from typing import cast, Any
@@ -296,25 +299,128 @@ def morsel_to_cookie(cookie: Morsel, /) -> Cookie:
     )
 
 
-def cookies_str_to_dict(cookies: str, /) -> dict[str, str]:
-    return dict(cookie.split("=", 1) for cookie in CRE_COOKIE_SEP_split(cookies) if cookie)
-
-
-def cookies_dict_to_str(
-    cookies: CookieJar | SimpleCookie | Mapping[str, Any] | Iterable[Any], 
+def cookies_to_dict(
+    cookies: str | CookieJar | SimpleCookie | Mapping[str, Any] | Iterable[Any], 
     /, 
-) -> str:
-    if isinstance(cookies, CookieJar):
-        cookie_it: Iterator[tuple[str, None | str]] = ((cookie.name, cookie.value) for cookie in cookies)
+    predicate: None | str | Container[str] | Callable[[str, Any], bool] = None, 
+) -> dict[str, str]:
+    if isinstance(predicate, str):
+        from urllib.parse import urlsplit
+        urlp = urlsplit(predicate)
+        def predicate(_, cookie, /) -> bool:
+            if isinstance(cookie, str):
+                return True
+            if (domain := getattr(cookie, "domain", "")) and not domain.endswith(urlp.netloc):
+                return False
+            if (url_path := urlp.path) and commonpath((url_path, (path := (getattr(cookie, "path", "") or "")))) != url_path:
+                return False
+            return True
+    elif isinstance(predicate, Container):
+        contains = predicate.__contains__
+        def predicate(name, _, /) -> bool:
+            return contains(name)
+    if isinstance(cookies, str):
+        cookie_it = (cookie.split("=", 1) for cookie in CRE_COOKIE_SEP_split(cookies))
+        if predicate is not None:
+            return {name: val for name, val in cookie_it if predicate(name, val)}
+        return dict(cookie_it)
+    elif isinstance(cookies, CookieJar):
+        return {
+            cookie.name: val for cookie in cookies 
+            if (val:=cookie.value) is not None and (
+                predicate is None or
+                predicate(cookie.name, cookie)
+            )
+        }
     elif isinstance(cookies, SimpleCookie):
-        cookie_it = ((name, morsel.value) for name, morsel in cookies.items())
+        return {
+            name: morsel.value for name, morsel in cookies.items()
+            if predicate is None or predicate(name, morsel)
+        }
     elif isinstance(cookies, Mapping):
-        cookie_it = ((name, cookie if isinstance(cookie, str) else cookie.value) for name, cookie in iter_items(cookies))
+        return {
+            name: cookie if isinstance(cookie, str) else cookie.value 
+            for name, cookie in iter_items(cookies)
+            if predicate is None or predicate(name, cookie)
+        }
     else:
         cookie_it = (
-            cookie[:2] if isinstance(cookie, tuple) else 
-                ((cookie.key if isinstance(cookie, Morsel) else getattr(cookie, "name", "")), getattr(cookie, "value", None)) 
-            for cookie in cookies
+            cookie[:2] if isinstance(cookie, tuple) else ((
+                cookie.key if isinstance(cookie, Morsel) else getattr(cookie, "name", "")), 
+                cookie, 
+            ) for cookie in cookies
+        )
+        if predicate is not None:
+            cookie_it = ((name, cookie) for name, cookie in cookie_it if predicate(name, cookie))
+        return {
+            key: val for key, cookie in cookie_it 
+            if key and (val := (
+                cookie if isinstance(cookie, str) else getattr(cookie, "value", None))
+            ) is not None
+        }
+
+
+def cookies_to_str(
+    cookies: str | CookieJar | SimpleCookie | Mapping[str, Any] | Iterable[Any], 
+    /, 
+    predicate: None | str | Container[str] | Callable[[str, Any], bool] = None, 
+) -> str:
+    if isinstance(predicate, str):
+        from urllib.parse import urlsplit
+        urlp = urlsplit(predicate)
+        def predicate(_, cookie, /) -> bool:
+            if isinstance(cookie, str):
+                return True
+            if (domain := getattr(cookie, "domain", "")) and not domain.endswith(urlp.netloc):
+                return False
+            if (url_path := urlp.path) and commonpath((url_path, (path := (getattr(cookie, "path", "") or "")))) != url_path:
+                return False
+            return True
+    elif isinstance(predicate, Container):
+        contains = predicate.__contains__
+        def predicate(name, _, /) -> bool:
+            return contains(name)
+    if isinstance(cookies, str):
+        if predicate is None:
+            return cookies
+        cookie_it: Iterator[tuple[str, Any]] = (
+            tuple(cookie.split("=", 1)) 
+            for cookie in CRE_COOKIE_SEP_split(cookies)
+        )
+        if predicate is not None:
+            cookie_it = ((name, val) for name, val in cookie_it if predicate(name, val))
+    elif isinstance(cookies, CookieJar):
+        cookie_it = (
+            (cookie.name, cookie.value) for cookie in cookies
+            if predicate is None or predicate(cookie.name, cookie)
+        )
+    elif isinstance(cookies, SimpleCookie):
+        cookie_it = (
+            (name, morsel.value) for name, morsel in cookies.items()
+            if predicate is None or predicate(name, morsel)
+        )
+    elif isinstance(cookies, Mapping):
+        cookie_it = (
+            (name, cookie if isinstance(cookie, str) else cookie.value) 
+            for name, cookie in iter_items(cookies)
+            if predicate is None or predicate(name, cookie)
+        )
+    else:
+        cookie_it = (
+            cookie[:2] if isinstance(cookie, tuple) else ((
+                cookie.key if isinstance(cookie, Morsel) else getattr(cookie, "name", "")), 
+                cookie
+            ) for cookie in cookies
+        )
+        if predicate is not None:
+            cookie_it = (
+                (name, cookie)
+                for name, cookie in cookie_it
+                if predicate(name, cookie)
+            )
+        cookie_it = (
+            (name, cookie if isinstance(cookie, str) else getattr(cookie, "value", None)) 
+            for name, cookie in cookie_it
         )
     return "; ".join(f"{key}={val}" for key, val in cookie_it if key and val is not None)
 
@@ -337,9 +443,13 @@ def extract_cookies[T: (SimpleCookie, CookieJar)](
     else:
         headers = response
     if isinstance(cookies, SimpleCookie):
-        cookiejar = CookieJar()
-        cookiejar.extract_cookies(response, Request(url)) # type: ignore
-        cookies.update((cookie.name, cookie_to_morsel(cookie)) for cookie in cookiejar)
+        for key, val in iter_items(headers):
+            key = key.lower()
+            if key in (b"set-cookie", b"set-cookie2"):
+                val = str(val, "latin-1")
+            elif key not in ("set-cookie", "set-cookie2"):
+                continue
+            cookies.load(val)
     else:
         cookies.extract_cookies(response, Request(url)) # type: ignore
     return cookies
@@ -354,9 +464,9 @@ def update_cookies[T: (SimpleCookie, CookieJar)](
         if isinstance(cookies2, SimpleCookie):
             morsels: Iterable[tuple[str, Morsel]] = cookies2.items()
         elif isinstance(cookies2, CookieJar):
-            morsels = ((cookie.name, to_morsel(cookie)) for cookie in cookies2)
+            morsels = ((cookie.name, to_morsel(cookie, name=cookie.name)) for cookie in cookies2)
         elif isinstance(cookies2, Mapping):
-            morsels = ((k, v if isinstance(v, Morsel) else to_morsel(v)) for k, v in iter_items(cookies2))
+            morsels = ((k, v if isinstance(v, Morsel) else to_morsel(v, name=k)) for k, v in iter_items(cookies2))
         else:
             morsels = ((c.key, c) for c in (c if isinstance(c, Morsel) else to_morsel(c) for c in cookies2))
         cookies1.update(morsels)
@@ -408,13 +518,17 @@ def iter_resp_cookies(resp, /) -> Iterator[tuple[str, None | str]]:
             return
         cookies = SimpleCookie()
         for k, v in iter_items(headers):
+            k = k.lower()
             if isinstance(k, Buffer):
                 k = str(k, "utf-8")
             if isinstance(v, Buffer):
                 v = str(v, "utf-8")
-            k = k.lower()
-            if k == "set-cookie":
+            if k == "set-cookie" or k == "set-cookie2":
                 cookies.load(v)
         for name, morsel in cookies.items():
             yield name, morsel.value
 
+
+cookies_str_to_dict = cookies_to_dict
+cookies_dict_to_str = cookies_to_str
+
